/* Author:  Mike Griswold*/
/* Supplementary materials for Practical Marginalized Multilevel Models*/



    
/** MMM for xover data DHLZ 2002 section 8.1 **/
/** Crossover Data Set Up **/
 data xover;
   input id Period Treatment y count @@;
   cards;
   1 0 1 1 22  1 1 0 1 22
   2 0 1 0  0  2 1 0 1  0
   3 0 1 1  6  3 1 0 0  6
   4 0 1 0  6  4 1 0 0  6
   5 0 0 1 18  5 1 1 1 18
   6 0 0 0  4  6 1 1 1  4
   7 0 0 1  2  7 1 1 0  2
   8 0 0 0  9  8 1 1 0  9
   ;
   run;
*proc print data=xover; run;

 /** Conditional Model **/
 TITLE "Logit-Normal GLMM";
 PROC NLMIXED data=xover;
    PARMS alpha0_c=.6 alpha1_c=.6 alpha2_c=-.3 tau=3;
    BOUNDS tau >=0;
    eta_c = alpha0_c + alpha1_c*Treatment + alpha2_c*Period + a;
    expeta_c = exp(eta_c);
    pi_c = expeta_c / (1+expeta_c);
    MODEL y ~ binary(pi_c);
    RANDOM a ~ NORMAL(0,tau*tau) SUBJECT=id;
    REPLICATE count;
 RUN;

 /**  LPN Marginal Model **/
 TITLE "Logit-Probit-Normal MMM";
 PROC NLMIXED data=xover qpoints=100;
    PARMS alpha0_m=.6 alpha1_m=.6 alpha2_m=-.3 tau=3;
    BOUNDS tau >=0;
    eta_m = alpha0_m + alpha1_m*Treatment + alpha2_m*Period;
    pi_m = 1 / (1 + exp(-eta_m));
		**-- LPN Method --**;
	    delta = sqrt(1+(tau*tau)) * probit(pi_m);
	    eta_c  =  delta + a;
	    pi_c = probnorm(eta_c);
    MODEL y ~ binary(pi_c);
    RANDOM a ~ NORMAL(0,tau*tau) SUBJECT=id;
    REPLICATE count;
 run;

 /**  LLB Marginal Model **/
 TITLE "Logit-Logit-Bridge MMM";
 PROC NLMIXED data=xover qpoints=100;
    PARMS alpha0_m=.6 alpha1_m=.6 alpha2_m=-.3 tau=3;
    BOUNDS tau >=0;
    eta_m = alpha0_m + alpha1_m*Treatment + alpha2_m*Period;
    pi_m = 1 / (1 + exp(-eta_m));
		**-- bridge method --**;
		pi=CONSTANT('pi');
		pnorm_asq = probnorm(asq);
		var_a = tau*tau;
		phi = (1+3*var_a/(pi*pi))**(-.5);
		Ginv_pnorm_asq = log(   sin(phi*pi*pnorm_asq) / 
								sin(phi*pi*(1-pnorm_asq))   ) / phi;
		expeta_c  =  exp( eta_m/phi + Ginv_pnorm_asq);
    	pi_c = expeta_c / (1+expeta_c); 
    MODEL y ~ binary(pi_c);
    RANDOM asq  ~ NORMAL(0,1) SUBJECT=id; 
    REPLICATE count;
 run;

 /**  JKB Marginal Model **/
 TITLE "Logit-Probit-Normal MMM (JKB fit)";
 %let c1=1.700437; 	*- Marginalization constant;
 PROC NLMIXED data=xover qpoints=100;
    PARMS alpha0_m=.6 alpha1_m=.6 alpha2_m=-.3 tau=3;
    BOUNDS tau >=0;
    eta_m = alpha0_m + alpha1_m*Treatment + alpha2_m*Period;
    pi_m = 1 / (1 + exp(-eta_m));
		**-- LPN model: JKB fit --**;
    	delta = sqrt(1+tau*tau) * eta_m / &c1;
		eta_c  =  delta + a;
    	pi_c = probnorm(eta_c); 
    MODEL y ~ binary(pi_c);
    RANDOM a ~ NORMAL(0,tau*tau) SUBJECT=id;
    REPLICATE count;
 run;

 /**  ZLA Marginal Model **/
 TITLE "Logit-Logit-Normal MMM (ZLA fit)";
 %let c1=1.700437; 	*- Marginalization constant;
 PROC NLMIXED data=xover qpoints=100;
    PARMS alpha0_m=.6 alpha1_m=.6 alpha2_m=-.3 tau=3;
    BOUNDS tau >=0;
    eta_m = alpha0_m + alpha1_m*Treatment + alpha2_m*Period;
    pi_m = 1 / (1 + exp(-eta_m));
		**-- LLN model: ZLA fit --**;
    	delta = sqrt(1+(tau*tau)/(&c1*&c1)) * eta_m;
		eta_c  =  delta + a;
    	pi_c = 1 / (1 + exp(-eta_c));
    MODEL y ~ binary(pi_c);
    RANDOM a ~ NORMAL(0,tau*tau) SUBJECT=id;
    REPLICATE count;
 run;

 /**  Page Marginal Model **/
 TITLE "Logit-Probit-Normal MMM (Page fit)";
 PROC NLMIXED data=xover qpoints=100;
    PARMS alpha0_m=.6 alpha1_m=.6 alpha2_m=-.3 tau=3;
    BOUNDS tau >=0;
    eta_m = alpha0_m + alpha1_m*Treatment + alpha2_m*Period;
    pi_m = 1 / (1 + exp(-eta_m));
		**-- LPN model: Page fit --**;
			a1 = 0.79880;
			a2 = 0.04417;
			B  = eta_m / (2*a1*a2); 
			up = ( -B + (B**2 + (4/27)*(1/a2)**3 )**(1/2) ) / 2;		
			tp = abs(up)**(1/3) * sign(up);
			sp = abs(B + up)**(1/3) * sign(B + up);
			fx_page = sp - tp;
    	delta = sqrt(1+tau*tau)*fx_page;
		eta_c  =  delta + a;
    	pi_c = probnorm(eta_c); 
    MODEL y ~ binary(pi_c);
    RANDOM a ~ NORMAL(0,tau*tau) SUBJECT=id;
    REPLICATE count;
 run;


/*Heagerty's specification does not preserve a readily interpretable
 conditional model, but we still put "Logit" in the Title of the code chunk for the conditional link.
 The approach requires iterative integration and inverting, so this code chunk
 differs from the above.  Be sure to have the pathname for ghq_wtsnodes.sas 
 correctly specified below; the resulting estimates will be in the work library
 in the dataset "Ests"*/ 
TITLE "Heagerty Logit-Logit-Normal";
/** MMM for xover data DHLZ 2002 section 8.1 **/

options nocenter nodate nonumber; 

title;

options nocenter nodate nonumber; 

/* same dataset but we rename some variables to keep following code more legible*/
data xover; 
  input id prd trt y count @@;
  cards;
  1 0 1 1 22  1 1 0 1 22
  2 0 1 0  0  2 1 0 1  0
  3 0 1 1  6  3 1 0 0  6
  4 0 1 0  6  4 1 0 0  6
  5 0 0 1 18  5 1 1 1 18
  6 0 0 0  4  6 1 1 1  4
  7 0 0 1  2  7 1 1 0  2
  8 0 0 0  9  8 1 1 0  9
  ;
  run;
*proc print; run;

  *%put %str(    ) Marginalizing...;
  **-- Include Guass-Hermite Quadrature Weights & Nodes macro --**;
  %inc "C:\Documents and Settings\student\My Documents\Downloads\ghq_wtsnodes.sas"; *give full path of ghq_wtsnodes.sas
  **-- Get Gauss/Hermite weights and nodes --**;
  %let gqnodes = 200;
  %ghq_wtsnodes(&gqnodes,dataname=wtsnodes,gnotes=F);
  proc transpose data=wtsnodes(keep=nodes) out=nodes(drop=_NAME_) prefix=node; run;
  proc transpose data=wtsnodes(keep=weights) out=wts(drop=_NAME_) prefix=wt; run;
  *proc print data=wtsnodes; run;
  *proc print data=nodes; run;
  *proc print data=wts; run;

  data xover; set xover;
   if _N_=1 then do; set nodes; set wts; end;
   run;
   *proc print data=xover; run;

%let r=&gqnodes;
**-- MMM --**;
PROC NLMIXED data=xover qpoints=&r noad noadscale; *corr; *itdetails trace;
  **-- INITIAL VALUES --**;
  PARMS alpha0_m   =  .6 
          alpha1_m   =  .6 
          alpha2_m   = -.3
      Ghalf =  5;
    **-- BOUNDS FOR VARiance components --**;
  BOUNDS Ghalf >=0;     
  **-- Programming Statements for log-likelihood --**;
  *- logistic marginal probability -*;
    eta_m = alpha0_m + alpha1_m*trt + alpha2_m*prd;
    pi_m = 1 / (1 + exp(-eta_m)); 

  **-- Solve for delta with Newtons Method and Gaussian Quadrature --**;
    array nodes[&r] node1-node&r;
    array wts[&r] wt1-wt&r;
    delta = eta_m;        *#- Initialize delta = Marginal predictor;
      delta_converge = 0;   *#- Convergence Indicator;
      iter = 0;       *#- Iteration Counter;

      *Stopping Criterion*;
    DELTA_TOLERANCE = 1e-8;
      DELTA_MAX_ITER  = 30;

      *##-- Newtons Method --##*;
    subj=id;
      do while( (iter < DELTA_MAX_ITER) & ^delta_converge );

        *##-- Use Gaussian Quadrature to Evaluate the integrals at the --##
        *##-- current estimate of the Expected conditional mean:       --##
        *##-- E.mu.c =  int{h(delta+sigma*z)*phi(z)*dz}                --##;

        *mu_c = 1/(1+exp( -(delta + Ghalf*nodes) )); *#- (vectorized) Conditional Mean at iteration i;
      
      E_mu_c = 0;
      d_E_mu_c_d_delta = 0;
      do k=1 to &r;
        mu_c_k = 1/(1+exp( -(delta + Ghalf*nodes[k]) )); *#- Conditional Mean at iteration i;
          E_mu_c = E_mu_c + wts[k]*mu_c_k;      *#- Expected Conditional Mean at iteration i;
          d_E_mu_c_d_delta = d_E_mu_c_d_delta + wts[k] * mu_c_k * (1 - mu_c_k); *#- derivative of[ E(mu.c) w.r.t delta at iteration i;
      end;

        *##- difference between marginal mean and integrated conditional mean, blown up by 
        *##- how much the conditional mean is changing relative to how much delta.c is changing.;
        delmu = ( E_mu_c - pi_m )/( d_E_mu_c_d_delta );  

        delta = delta - delmu;  *#-- update current estimate of delta;

        delta_converge = abs(delmu) < DELTA_TOLERANCE;
        iter = iter + 1;        *#- increment iteration counter;

      *if subj=1 then put subj iter delta_converge pi_m E_mu_c eta_m delta delmu;
      end;

  eta_c = delta + a;
  pi_c = 1 / (1 + exp(-eta_c)); 
    *- log-likelihood -*;
    MODEL y ~ binary(pi_c);
  **-- Random Effects --**;
    RANDOM a ~ NORMAL(0,Ghalf*Ghalf) SUBJECT=id; 
  REPLICATE count;
    ODS OUTPUT ParameterEstimates=ests;
run;

data ests&r; set ests; run;

proc print data=ests; run;


title GQ points=200;
proc print data=ests200; run;







